/*
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *                    Version 2, December 2004
 *
 * Copyright (C) 2013 Willy "z3bra" Goiffon <willy@mailoo.org>
 *
 * Everyone is permitted to copy and distribute verbatim or modified
 * copies of this license document, and changing it is allowed as long
 * as the name is changed.
 *
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *
 *  0. You just DO WHAT THE FUCK YOU WANT TO.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <pthread.h>
#include <sys/inotify.h>

#define BUF_LEN 512 /* buffer size for input */
#define DIR_LEN 255 /* directory lengh in char */
#define EVT_LEN (128 * (EVENT_SIZE + 16))
#define EVENT_SIZE (sizeof (struct inotify_event))

/*
 * Represent a ii channel
 *  - index     : a unique index that represent the channel
 *  - alias     : a name given to the channel
 *  - input     : the path to its 'in' file
 */
typedef struct channel {
    int     index;
    char    alias[32];
    char    input[DIR_LEN];
    bool    connected;
} chan_t;

int add_channel(char *alias);
void del_channel(char *alias);
char *build_path(char *str, char *root, char *channel);
chan_t *chan_init(chan_t *channels);
chan_t *chan_get_by_index (int index);
chan_t *chan_get_by_alias (char *alias);
int chan_switch_to_index (int index);
int chan_switch_to_alias (char *alias);
void *inotify_handler(void *args);
void  inotify_event_parser(int wd);
int parse_command (char *buffer);
void usage();
void run(char *root);

int g_fildes = 0;       /* global file descriptor */
int g_index = 0;        /* unique index for channels */
char g_prompt = ']';    /* character used for prompt */

void usage()
{
    fputs(
            "usage: ptii [-h] [-i <irc_dir>] [-p <char>]\n"
            "\t-h            : display this help\n"
            "\t-i <irc_dir>  : use <irc_dir> instead of current directory\n"
            "\t-p <char>     : use <char> as the prompt\n"
            ,stdout);

    exit (0);
}

int add_channel (char *alias)
{
    return ++g_index;
}

void *inotify_handler(void *args)
{
    int fd = 0;
    int wd = 0;
    char *rootdir = NULL;

    rootdir = (char*) args;

    fd = inotify_init();
    if (fd < 0) { perror("inotify_init"); }

    wd = inotify_add_watch (fd, rootdir, IN_CREATE);
    if (wd < 0) { perror("inotify_add_watch"); exit(errno); }

    printf("watching %s ..\n", rootdir);

    for (;;) {
        inotify_event_parser(fd);
    }

    return NULL;
}

void inotify_event_parser(int fd)
{
    int len, i=0;
    char *events = NULL;
    struct inotify_event *ev;

    events = calloc(1, EVT_LEN); /* enough size for 128 events */

    len = read (fd, events,  EVT_LEN);

    if (len <= 0 ) {
        perror("read");
    }

    while (i < len) {

        ev = (struct inotify_event *) &events[i];

        /* Let's just handle directories here */
        if (ev->mask & IN_ISDIR && ev->len) {
            /* ii has just created a directory */
            if (ev->mask & IN_CREATE) {
                add_channel(ev->name);
                printf ("%s created\n", ev->name);
            }
        }

        i += EVENT_SIZE + ev->len;
    }
}

int main(int argc, char **argv)
{
    int i;
    pthread_t th;
    char cwd[DIR_LEN];

    if (getcwd(cwd, DIR_LEN) == NULL) {
        perror("getcwd");
        exit(errno);
    }

    if ((argc == 2 && argv[1][0] == '-' && argv[1][1] == 'h')) usage();

    for(i = 1; (i + 1 < argc) && (argv[i][0] == '-'); i++) {
        switch (argv[i][1]) {
            case 'i': strncpy(cwd, argv[++i], DIR_LEN); break;
            case 'p': g_prompt = argv[++i][0]; break;
        }
    }

    pthread_create (&th, NULL, inotify_handler, (void*)cwd);

/*    pthread_cancel(th); */
    pthread_join (th, NULL);

    return EXIT_SUCCESS;
}
